@@ -389,7 +389,7 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
             plugin_instance = self.plugin_registry[plugin_name]
             plugin_instance.register_plugin_preferences(self)
 
-    def delete_plugin(self, plugin_name: str) -> bool:
+    def delete_plugin(self, plugin_name: str, teardown: bool = True) -> bool:
         """
         Remove and delete a plugin from the registry by its name.
 
@@ -397,6 +397,9 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
         ----------
         plugin_name: str
             Name of the plugin to delete.
+        teardown: bool
+            True if the teardown notification to other plugins should be sent
+            when deleting the plugin, False otherwise.
 
         Returns
         -------
@@ -404,6 +407,7 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
             True if the registry was able to teardown and remove the plugin.
             False otherwise.
         """
+        logger.debug(f'Deleting plugin {plugin_name}')
         plugin_instance = self.plugin_registry[plugin_name]
 
         # Determine if plugin can be closed
@@ -417,11 +421,34 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
             return False
 
         if isinstance(plugin_instance, SpyderPluginV2):
-            # Disconnect plugin from other plugins
-            self._teardown_plugin(plugin_name)
+            # Cleanly delete plugin widgets. This avoids segfautls with
+            # PyQt 5.15
+            if isinstance(plugin_instance, SpyderDockablePlugin):
+                try:
+                    plugin_instance.get_widget().close()
+                    plugin_instance.get_widget().deleteLater()
+                except RuntimeError:
+                    pass
+            else:
+                container = plugin_instance.get_container()
+                if container:
+                    try:
+                        container.close()
+                        container.deleteLater()
+                    except RuntimeError:
+                        pass
 
-            # Disconnect depending plugins from the plugin to delete
-            self._notify_plugin_teardown(plugin_name)
+            # Delete plugin
+            try:
+                plugin_instance.deleteLater()
+            except RuntimeError:
+                pass
+            if teardown:
+                # Disconnect plugin from other plugins
+                self._teardown_plugin(plugin_name)
+
+                # Disconnect depending plugins from the plugin to delete
+                self._notify_plugin_teardown(plugin_name)
 
             # Remove the plugin from the main window (if graphical)
             if isinstance(plugin_instance, SpyderDockablePlugin):
@@ -435,10 +462,18 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
                 plugin_instance.close_window()
 
             # Perform plugin closure tasks
-            plugin_instance.on_close(True)
+            try:
+                plugin_instance.on_close(True)
+            except RuntimeError:
+                pass
         elif isinstance(plugin_instance, SpyderPlugin):
-            # Disconnect depending plugins from the plugin to delete
-            self._notify_plugin_teardown(plugin_name)
+            try:
+                plugin_instance.deleteLater()
+            except RuntimeError:
+                pass
+            if teardown:
+                # Disconnect depending plugins from the plugin to delete
+                self._notify_plugin_teardown(plugin_name)
             if isinstance(plugin_instance, SpyderPluginWidget):
                 # Save if plugin was undocked to restore it the next time.
                 if plugin_instance._undocked_window:
@@ -454,6 +489,10 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
         # Delete plugin from the registry and auxiliary structures
         self.plugin_dependents.pop(plugin_name, None)
         self.plugin_dependencies.pop(plugin_name, None)
+        if plugin_instance.CONF_FILE:
+            # This must be done after on_close() so that plugins can modify
+            # their (external) config therein.
+            CONF.unregister_plugin(plugin_instance)
 
         for plugin in self.plugin_dependents:
             all_plugin_dependents = self.plugin_dependents[plugin]
@@ -477,6 +516,7 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
 
         # Remove the plugin from the registry
         self.plugin_registry.pop(plugin_name)
+
         return True
 
     def delete_all_plugins(self, excluding: Optional[Set[str]] = None,
@@ -508,7 +548,21 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
             if plugin_name not in excluding:
                 plugin_instance = self.plugin_registry[plugin_name]
                 if isinstance(plugin_instance, SpyderPlugin):
-                    can_close &= self.delete_plugin(plugin_name)
+                    can_close &= self.delete_plugin(
+                        plugin_name, teardown=False)
+                    if not can_close and not close_immediately:
+                        break
+
+        if not can_close:
+            return False
+
+        # Delete Spyder 4 internal plugins
+        for plugin_name in set(self.internal_plugins):
+            if plugin_name not in excluding:
+                plugin_instance = self.plugin_registry[plugin_name]
+                if isinstance(plugin_instance, SpyderPlugin):
+                    can_close &= self.delete_plugin(
+                        plugin_name, teardown=False)
                     if not can_close and not close_immediately:
                         break
 
@@ -520,30 +574,21 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
             if plugin_name not in excluding:
                 plugin_instance = self.plugin_registry[plugin_name]
                 if isinstance(plugin_instance, SpyderPluginV2):
-                    can_close &= self.delete_plugin(plugin_name)
+                    can_close &= self.delete_plugin(
+                        plugin_name, teardown=False)
                     if not can_close and not close_immediately:
                         break
 
         if not can_close and not close_immediately:
             return False
 
-        # Delete Spyder 4 internal plugins
-        for plugin_name in set(self.internal_plugins):
-            if plugin_name not in excluding:
-                plugin_instance = self.plugin_registry[plugin_name]
-                if isinstance(plugin_instance, SpyderPlugin):
-                    can_close &= self.delete_plugin(plugin_name)
-                    if not can_close and not close_immediately:
-                        break
-
-        if not can_close:
-            return False
-
+        # Delete Spyder 5 internal plugins
         for plugin_name in set(self.internal_plugins):
             if plugin_name not in excluding:
                 plugin_instance = self.plugin_registry[plugin_name]
                 if isinstance(plugin_instance, SpyderPluginV2):
-                    can_close &= self.delete_plugin(plugin_name)
+                    can_close &= self.delete_plugin(
+                        plugin_name, teardown=False)
                     if not can_close and not close_immediately:
                         break
 
@@ -649,7 +694,7 @@ class SpyderPluginRegistry(QObject, PreferencesAdapter):
 
         try:
             self.sig_plugin_ready.disconnect()
-        except TypeError:
+        except (TypeError, RuntimeError):
             # Omit failures if there are no slots connected
             pass
 
