@@ -13,11 +13,8 @@ Client widget for the IPython Console.
 This is the widget used on all its tabs.
 """
 
-
 # Standard library imports.
-# Fix for spyder-ide/spyder#1356.
-from __future__ import absolute_import
-
+import logging
 import os
 import os.path as osp
 import re
@@ -31,7 +28,8 @@ from qtpy.QtWidgets import (QMessageBox, QVBoxLayout, QWidget)
 # Local imports
 from spyder.api.translations import get_translation
 from spyder.api.widgets.mixins import SpyderWidgetMixin
-from spyder.config.base import get_module_source_path, running_under_pytest
+from spyder.config.base import (
+    get_home_dir, get_module_source_path, running_under_pytest)
 from spyder.utils.icon_manager import ima
 from spyder.utils import sourcecode
 from spyder.utils.image_path_manager import get_image_path
@@ -39,17 +37,16 @@ from spyder.utils.installers import InstallerIPythonKernelError
 from spyder.utils.encoding import get_coding
 from spyder.utils.environ import RemoteEnvDialog
 from spyder.utils.palette import QStylePalette
-from spyder.utils.programs import get_temp_dir
 from spyder.utils.qthelpers import add_actions, DialogManager
 from spyder.py3compat import to_text_string
-from spyder.plugins.ipythonconsole.utils.stdfile import StdFile
 from spyder.plugins.ipythonconsole.widgets import ShellWidget
 from spyder.widgets.collectionseditor import CollectionsEditor
 from spyder.widgets.mixins import SaveHistoryMixin
 
 
-# Localization
+# Localization and logging
 _ = get_translation('spyder')
+logger = logging.getLogger(__name__)
 
 # -----------------------------------------------------------------------------
 # Templates
@@ -107,9 +104,10 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                  ask_before_restart=True,
                  ask_before_closing=False,
                  css_path=None,
-                 configuration=None,
                  handlers={},
-                 std_dir=None):
+                 stderr_obj=None,
+                 stdout_obj=None,
+                 fault_obj=None):
         super(ClientWidget, self).__init__(parent)
         SaveHistoryMixin.__init__(self, history_filename)
 
@@ -132,7 +130,6 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.options_button = options_button
         self.history = []
         self.allow_rename = True
-        self.std_dir = std_dir
         self.is_error_shown = False
         self.error_text = None
         self.restart_thread = None
@@ -152,8 +149,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             is_external_kernel=is_external_kernel,
             is_spyder_kernel=is_spyder_kernel,
             handlers=handlers,
-            local_kernel=True,
-            configuration=configuration)
+            local_kernel=True
+        )
         self.infowidget = self.container.infowidget
         self.blank_page = self._create_blank_page()
         self.loading_page = self._create_loading_page()
@@ -170,7 +167,8 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.layout = QVBoxLayout()
         self.layout.setContentsMargins(0, 0, 0, 0)
         self.layout.addWidget(self.shellwidget)
-        self.layout.addWidget(self.infowidget)
+        if self.infowidget is not None:
+            self.layout.addWidget(self.infowidget)
         self.setLayout(self.layout)
 
         # --- Exit function
@@ -180,33 +178,31 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         self.dialog_manager = DialogManager()
 
         # --- Standard files handling
-        self.stderr_obj = None
-        self.stdout_obj = None
-        self.fault_obj = None
+        self.stderr_obj = stderr_obj
+        self.stdout_obj = stdout_obj
+        self.fault_obj = fault_obj
         self.std_poll_timer = None
-        if not self.is_external_kernel:
-            # Cannot create std files for external kernels
-            self.stderr_obj = StdFile(self.std_filename('.stderr'))
-            self.stdout_obj = StdFile(self.std_filename('.stdout'))
+        if self.stderr_obj is not None or self.stdout_obj is not None:
             self.std_poll_timer = QTimer(self)
             self.std_poll_timer.timeout.connect(self.poll_std_file_change)
             self.std_poll_timer.setInterval(1000)
             self.std_poll_timer.start()
             self.shellwidget.executed.connect(self.poll_std_file_change)
-        if self.hostname is None:
-            # Cannot read file that is not on this computer
-            self.fault_obj = StdFile(self.std_filename('.fault'))
+
+        self.start_successful = False
 
     def __del__(self):
         """Close threads to avoid segfault."""
         if (self.restart_thread is not None
                 and self.restart_thread.isRunning()):
+            self.restart_thread.quit()
             self.restart_thread.wait()
 
     # ----- Private methods ---------------------------------------------------
-    def _before_prompt_is_ready(self):
-        """Configure shellwidget before kernel is connected."""
-        self._show_loading_page()
+    def _before_prompt_is_ready(self, show_loading_page=True):
+        """Configuration before kernel is connected."""
+        if show_loading_page:
+            self._show_loading_page()
         self.shellwidget.sig_prompt_ready.connect(
             self._when_prompt_is_ready)
         # If remote execution, the loading page should be hidden as well
@@ -215,6 +211,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def _when_prompt_is_ready(self):
         """Configuration after the prompt is shown."""
+        self.start_successful = True
         # To hide the loading page
         self._hide_loading_page()
 
@@ -224,6 +221,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         # To show if special console is valid
         self._check_special_console_error()
 
+        # Set the initial current working directory
+        self._set_initial_cwd()
+
         self.shellwidget.sig_prompt_ready.disconnect(
             self._when_prompt_is_ready)
         self.shellwidget.sig_remote_execute.disconnect(
@@ -256,16 +256,18 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def _show_loading_page(self):
         """Show animation while the kernel is loading."""
-        self.shellwidget.hide()
-        self.infowidget.show()
-        self.info_page = self.loading_page
-        self.set_info_page()
+        if self.infowidget is not None:
+            self.shellwidget.hide()
+            self.infowidget.show()
+            self.info_page = self.loading_page
+            self.set_info_page()
 
     def _hide_loading_page(self):
         """Hide animation shown while the kernel is loading."""
-        self.infowidget.hide()
-        self.info_page = self.blank_page
-        self.set_info_page()
+        if self.infowidget is not None:
+            self.infowidget.hide()
+            self.info_page = self.blank_page
+            self.set_info_page()
         self.shellwidget.show()
 
     def _read_stderr(self):
@@ -323,12 +325,14 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
         We also ignore errors about comms, which are irrelevant.
         """
+        if self.start_successful:
+            return False
         stderr = self.stderr_obj.get_contents()
         if not stderr:
             return False
-        # There is an error. If it is only about comms, ignore.
+        # There is an error. If it is benign, ignore.
         for line in stderr.splitlines():
-            if line and 'No such comm' not in line:
+            if line and not self.is_benign_error(line):
                 return True
         return False
 
@@ -348,21 +352,55 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         page_control.sig_show_find_widget_requested.connect(
             self.container.find_widget.show)
 
-    # ----- Public API --------------------------------------------------------
-    def std_filename(self, extension):
-        """Filename to save kernel output."""
-        file = None
-        if self.connection_file is not None:
-            file = self.kernel_id + extension
-            if self.std_dir is not None:
-                file = osp.join(self.std_dir, file)
-            else:
-                try:
-                    file = osp.join(get_temp_dir(), file)
-                except (IOError, OSError):
-                    file = None
-        return file
+    def _set_initial_cwd(self):
+        """Set initial cwd according to preferences."""
+        logger.debug("Setting initial working directory")
+        cwd_path = get_home_dir()
+        project_path = self.container.get_active_project_path()
 
+        # This is for the first client
+        if self.id_['int_id'] == '1':
+            if self.get_conf(
+                'startup/use_project_or_home_directory',
+                section='workingdir'
+            ):
+                cwd_path = get_home_dir()
+                if project_path is not None:
+                    cwd_path = project_path
+            elif self.get_conf(
+                'startup/use_fixed_directory',
+                section='workingdir'
+            ):
+                cwd_path = self.get_conf(
+                    'startup/fixed_directory',
+                    default=get_home_dir(),
+                    section='workingdir'
+                )
+        else:
+            # For new clients
+            if self.get_conf(
+                'console/use_project_or_home_directory',
+                section='workingdir'
+            ):
+                cwd_path = get_home_dir()
+                if project_path is not None:
+                    cwd_path = project_path
+            elif self.get_conf('console/use_cwd', section='workingdir'):
+                cwd_path = self.container.get_working_directory()
+            elif self.get_conf(
+                'console/use_fixed_directory',
+                section='workingdir'
+            ):
+                cwd_path = self.get_conf(
+                    'console/fixed_directory',
+                    default=get_home_dir(),
+                    section='workingdir'
+                )
+
+        if osp.isdir(cwd_path):
+            self.shellwidget.set_cwd(cwd_path)
+
+    # ----- Public API --------------------------------------------------------
     @property
     def kernel_id(self):
         """Get kernel id."""
@@ -390,29 +428,33 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
     def poll_std_file_change(self):
         """Check if the stderr or stdout file just changed."""
         self.shellwidget.call_kernel().flush_std()
-        stderr = self.stderr_obj.poll_file_change()
         starting = self.shellwidget._starting
-        if stderr:
-            if self.shellwidget.isHidden():
-                # Avoid printing the same thing again
-                if self.error_text != '<tt>%s</tt>' % stderr:
-                    full_stderr = self.stderr_obj.get_contents()
-                    self.show_kernel_error('<tt>%s</tt>' % full_stderr)
-            if starting:
-                self.shellwidget.banner = (
-                    stderr + '\n' + self.shellwidget.banner)
-            else:
-                self.shellwidget._append_plain_text(
-                    '\n' + stderr, before_prompt=True)
+        if self.stderr_obj is not None:
+            stderr = self.stderr_obj.poll_file_change()
+            if stderr:
+                if self.is_benign_error(stderr):
+                    return
+                if self.shellwidget.isHidden():
+                    # Avoid printing the same thing again
+                    if self.error_text != '<tt>%s</tt>' % stderr:
+                        full_stderr = self.stderr_obj.get_contents()
+                        self.show_kernel_error('<tt>%s</tt>' % full_stderr)
+                if starting:
+                    self.shellwidget.banner = (
+                        stderr + '\n' + self.shellwidget.banner)
+                else:
+                    self.shellwidget._append_plain_text(
+                        '\n' + stderr, before_prompt=True)
 
-        stdout = self.stdout_obj.poll_file_change()
-        if stdout:
-            if starting:
-                self.shellwidget.banner = (
-                    stdout + '\n' + self.shellwidget.banner)
-            else:
-                self.shellwidget._append_plain_text(
-                    '\n' + stdout, before_prompt=True)
+        if self.stdout_obj is not None:
+            stdout = self.stdout_obj.poll_file_change()
+            if stdout:
+                if starting:
+                    self.shellwidget.banner = (
+                        stdout + '\n' + self.shellwidget.banner)
+                else:
+                    self.shellwidget._append_plain_text(
+                        '\n' + stdout, before_prompt=True)
 
     def configure_shellwidget(self, give_focus=True):
         """Configure shellwidget after kernel is connected."""
@@ -494,12 +536,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
         """Show kernel initialization errors in infowidget."""
         self.error_text = error
 
-        # Filter out benign errors
-        if "http://bugs.python.org/issue1666807" in error:
-            # See spyder-ide/spyder#16828
-            return
-        if "https://bugs.python.org/issue1180193" in error:
-            # See spyder-ide/spyder#16927
+        if self.is_benign_error(error):
             return
 
         InstallerIPythonKernelError(error)
@@ -522,13 +559,40 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             error=error)
 
         # Show error
-        self.set_info_page()
-        self.shellwidget.hide()
-        self.infowidget.show()
+        if self.infowidget is not None:
+            self.set_info_page()
+            self.shellwidget.hide()
+            self.infowidget.show()
 
         # Tell the client we're in error mode
         self.is_error_shown = True
 
+        # Stop shellwidget
+        self.shellwidget.shutdown()
+        self.remove_std_files(is_last_client=False)
+
+    def is_benign_error(self, error):
+        """Decide if an error is benign in order to filter it."""
+        benign_errors = [
+            # Error when switching from the Qt5 backend to the Tk one.
+            # See spyder-ide/spyder#17488
+            "KeyboardInterrupt caught in kernel",
+            "QSocketNotifier: Multiple socket notifiers for same socket",
+            # Error when switching from the Tk backend to the Qt5 one.
+            # See spyder-ide/spyder#17488
+            "Tcl_AsyncDelete async handler deleted by the wrong thread",
+            "error in background error handler:",
+            "    while executing",
+            '"::tcl::Bgerror',
+            # Avoid showing this warning because it was up to the user to
+            # disable secure writes.
+            "WARNING: Insecure writes have been enabled via environment",
+            # Old error
+            "No such comm"
+        ]
+
+        return any([err in error for err in benign_errors])
+
     def get_name(self):
         """Return client name"""
         if self.given_name is None:
@@ -583,11 +647,16 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
     def shutdown(self, is_last_client):
         """Shutdown connection and kernel if needed."""
         self.dialog_manager.close_all()
-        self.remove_std_files(is_last_client)
+        if (self.restart_thread is not None
+                and self.restart_thread.isRunning()):
+            self.restart_thread.finished.disconnect()
+            self.restart_thread.quit()
+            self.restart_thread.wait()
         shutdown_kernel = (
             is_last_client and not self.is_external_kernel
             and not self.is_error_shown)
         self.shellwidget.shutdown(shutdown_kernel)
+        self.remove_std_files(shutdown_kernel)
 
     def interrupt_kernel(self):
         """Interrupt the associanted Spyder kernel if it's running"""
@@ -620,8 +689,9 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                 running_under_pytest() or
                 not self.ask_before_restart):
             if sw.kernel_manager:
-                if self.infowidget.isVisible():
-                    self.infowidget.hide()
+                if self.infowidget is not None:
+                    if self.infowidget.isVisible():
+                        self.infowidget.hide()
 
                 if self._abort_kernel_restart():
                     sw.spyder_kernel_comm.close()
@@ -636,13 +706,16 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
                 sw.kernel_manager.stop_restarter()
                 sw.kernel_manager.autorestart = False
 
+                # Reconfigure client before the new kernel is connected again.
+                self._before_prompt_is_ready(show_loading_page=False)
+
                 # Create and run restarting thread
                 if (self.restart_thread is not None
                         and self.restart_thread.isRunning()):
                     self.restart_thread.finished.disconnect()
-                    self.restart_thread.terminate()
+                    self.restart_thread.quit()
                     self.restart_thread.wait()
-                self.restart_thread = QThread()
+                self.restart_thread = QThread(None)
                 self.restart_thread.run = self._restart_thread_main
                 self.restart_thread.error = None
                 self.restart_thread.finished.connect(
@@ -689,7 +762,12 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
             # Reset Pdb state and reopen comm
             sw._pdb_in_loop = False
             sw.spyder_kernel_comm.remove()
-            sw.spyder_kernel_comm.open_comm(sw.kernel_client)
+            try:
+                sw.spyder_kernel_comm.open_comm(sw.kernel_client)
+            except AttributeError:
+                # An error occurred while opening our comm channel.
+                # Aborting!
+                return
 
             # Start autorestart mechanism
             sw.kernel_manager.autorestart = True
@@ -852,7 +930,7 @@ class ClientWidget(QWidget, SaveHistoryMixin, SpyderWidgetMixin):
 
     def set_info_page(self):
         """Set current info_page."""
-        if self.info_page is not None:
+        if self.infowidget is not None and self.info_page is not None:
             self.infowidget.setHtml(
                 self.info_page,
                 QUrl.fromLocalFile(self.css_path)
