@@ -21,6 +21,7 @@ from pygments.lexer import bygroups
 from pygments.token import Keyword, Operator, Text
 from pygments.util import ClassNotFound
 from qtconsole.rich_jupyter_widget import RichJupyterWidget
+from qtpy.QtCore import QEvent
 from qtpy.QtGui import QTextCursor
 
 # Local imports
@@ -74,14 +75,6 @@ class DebuggingHistoryWidget(RichJupyterWidget):
         super(DebuggingHistoryWidget, self).__init__(*args, **kwargs)
 
     # --- Public API --------------------------------------------------
-    def shutdown(self):
-        """Shutdown the widget"""
-        try:
-            self._pdb_history_file.save_thread.stop()
-            self._pdb_history_file.db.close()
-        except AttributeError:
-            pass
-
     def new_history_session(self):
         """Start a new history session."""
         self._pdb_history_input_number = 0
@@ -203,10 +196,9 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
             r'[ \t]*\(*IPdb \[\d+\]\)*: |' +
             r'[ \t]*In \[\d+\]: |[ \t]*\ \ \ \.\.\.+: )')
 
-
     # --- Public API --------------------------------------------------
 
-    def will_close(self, externally_managed):
+    def shutdown(self):
         """
         Close the save thread and database file.
         """
@@ -434,6 +426,21 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
         if pdb_state is not None and isinstance(pdb_state, dict):
             self.refresh_from_pdb(pdb_state)
 
+    def show_pdb_output(self, text):
+        """Show Pdb output."""
+        self._append_plain_text(self.output_sep, before_prompt=True)
+        prompt = self._current_out_prompt()
+        self._append_html(
+            '<span class="out-prompt">%s</span>' % prompt,
+            before_prompt=True
+        )
+        # If the repr is multiline, make sure we start on a new line,
+        # so that its lines are aligned.
+        if "\n" in text and not self.output_sep.endswith("\n"):
+            self._append_plain_text('\n', before_prompt=True)
+        self._append_plain_text(text + self.output_sep2, before_prompt=True)
+        self._append_plain_text('\n', before_prompt=True)
+
     def get_pdb_last_step(self):
         """Get last pdb step retrieved from a Pdb session."""
         fname, lineno = self._pdb_frame_loc
@@ -474,6 +481,14 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
             prompt = "({})".format(prompt)
         return prompt + ": "
 
+    def _current_out_prompt(self):
+        """Get current out prompt."""
+        prompt = "Out\u00A0\u00A0[{}]".format(self._pdb_history_input_number)
+        for i in range(self._pdb_in_loop - 1):
+            # Add recursive debugger prompt
+            prompt = "({})".format(prompt)
+        return prompt + ": "
+
     def _handle_kernel_info_reply(self, rep):
         """Handle kernel info replies."""
         super(DebuggingWidget, self)._handle_kernel_info_reply(rep)
@@ -578,9 +593,6 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
 
     def pdb_input(self, prompt, password=None):
         """Get input for a command."""
-        if self._hidden:
-            raise RuntimeError(
-                'Request for pdb input during hidden execution.')
 
         # Replace with numbered prompt
         prompt = self._current_prompt()
@@ -626,14 +638,16 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
             return
 
     # --- Private API (overrode by us) ----------------------------------------
-    def _show_prompt(self, prompt=None, html=False, newline=True):
+    def _show_prompt(self, prompt=None, html=False, newline=True,
+                     separator=True):
         """
         Writes a new prompt at the end of the buffer.
         """
         if prompt == self._pdb_prompt[0]:
             html = True
             prompt = '<span class="in-prompt">%s</span>' % prompt
-        super(DebuggingWidget, self)._show_prompt(prompt, html, newline)
+        super(DebuggingWidget, self)._show_prompt(prompt, html, newline,
+                                                  separator)
 
     def _event_filter_console_keypress(self, event):
         """Handle Key_Up/Key_Down while debugging."""
@@ -662,3 +676,16 @@ class DebuggingWidget(DebuggingHistoryWidget, SpyderConfigurationAccessor):
         else:
             return super(DebuggingWidget, self)._register_is_complete_callback(
                 source, callback)
+
+    # ---- Qt methods ---------------------------------------------------------
+    def eventFilter(self, obj, event):
+        # When using PySide, it can happen that "event" is of type QWidgetItem
+        # (reason unknown). This causes an exception in eventFilter() in
+        # console_widget.py in the QtConsole package: Therein event.type() is
+        # accessed which fails due to an AttributeError. Catch this here and
+        # ignore the event.
+        if not isinstance(event, QEvent):
+            # Note for debugging: event.layout() or event.widget() SEGFAULTs
+            return True
+
+        return super().eventFilter(obj, event)
